VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "howto_find_circle_tangents"
   ClientHeight    =   3090
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3090
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Const NAN As Single = 1E+20

' The last point clicked.
Private ClickedX As Integer
Private ClickedY As Integer

Private Sub Form_Load()
    ClickedX = -1
    ClickedY = -1
    Me.ScaleMode = vbPixels
End Sub

Private Sub Form_Resize()
    Me.Refresh
End Sub

' Save the clicked point.
Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Save the point.
    ClickedX = X
    ClickedY = Y

    ' Redraw.
    Me.Refresh
End Sub

' Draw the circle, point, and tangent lines.
Private Sub Form_Paint()
Dim radius As Single
Dim cx As Single
Dim cy As Single
Dim px1 As Single
Dim py1 As Single
Dim px2 As Single
Dim py2 As Single

    ' Draw the circle.
    If (ScaleWidth > ScaleHeight) Then
        radius = Me.ScaleHeight / 6
    Else
        radius = Me.ScaleWidth / 6
    End If
    cx = Me.ScaleWidth - radius - 20
    cy = Me.ScaleHeight - radius - 20
    
    Me.Cls
    Me.FillColor = RGB(128, 128, 255)
    Me.FillStyle = vbFSSolid
    Me.Circle (cx, cy), radius, vbBlue

    If (ClickedX < 0) Then Exit Sub

    ' Find the tangents.
    If FindTangents(cx, cy, radius, ClickedX, ClickedY, _
        px1, py1, px2, py2) _
    Then
        Me.Line (ClickedX, ClickedY)-(px1, py1), vbGreen
        Me.Line (ClickedX, ClickedY)-(px2, py2), vbGreen
    End If

    ' Draw the point.
    Me.FillColor = RGB(255, 128, 128)
    Me.Circle (ClickedX, ClickedY), 3, vbRed
End Sub

' Find the tangent points for this circle and external point.
' Return true if we find the tangents, false if the point is
' inside the circle.
Private Function FindTangents(ByVal cx As Single, ByVal cy As Single, _
    ByVal radius As Single, ByVal px As Single, ByVal py As Single, _
    ByRef px1 As Single, ByRef py1 As Single, _
    ByRef px2 As Single, ByRef py2 As Single) As Boolean
Dim dx As Double
Dim dy As Double
Dim D_squared As Double
Dim L As Double

    ' Find the distance squared from the
    ' external point to the circle's center.
    dx = cx - px
    dy = cy - py
    D_squared = dx * dx + dy * dy
    If (D_squared < radius * radius) Then
        px1 = -1
        FindTangents = False
        Exit Function
    End If

    ' Find the distance from the external point
    ' to the tangent points.
    L = Sqr(D_squared - radius * radius)

    ' Find the points of intersection between
    ' the original circle and the circle with
    ' center external_point and radius dist.
    FindCircleCircleIntersections _
        cx, cy, radius, _
        px, py, CSng(L), _
        px1, py1, px2, py2

    FindTangents = True
End Function

' Find the points where the two circles intersect.
Private Function FindCircleCircleIntersections( _
    ByVal cx0 As Single, ByVal cy0 As Single, _
    ByVal radius0 As Single, _
    ByVal cx1 As Single, ByVal cy1 As Single, _
    ByVal radius1 As Single, _
    ByRef intersectionx1 As Single, _
    ByRef intersectiony1 As Single, _
    ByRef intersectionx2 As Single, _
    ByRef intersectiony2 As Single) As Integer
Dim dx, dy As Single
Dim dist, a, h, cx2, cy2 As Double

    ' Find the distance between the centers.
    dx = cx0 - cx1
    dy = cy0 - cy1
    dist = Sqr(dx * dx + dy * dy)

    ' See how many solutions there are.
    If (dist > radius0 + radius1) Then
        ' No solutions, the circles are too far apart.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
        Exit Function
    ElseIf (dist < Math.Abs(radius0 - radius1)) Then
        ' No solutions, one circle contains the other.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
    ElseIf ((dist = 0) And (radius0 = radius1)) Then
        ' No solutions, the circles coincide.
        intersectionx1 = NAN
        intersectiony1 = NAN
        intersectionx2 = NAN
        intersectiony2 = NAN
        FindCircleCircleIntersections = 0
    Else
        ' Find a and h.
        a = (radius0 * radius0 - _
            radius1 * radius1 + dist * dist) / (2 * dist)
        h = Sqr(radius0 * radius0 - a * a)

        ' Find P2.
        cx2 = cx0 + a * (cx1 - cx0) / dist
        cy2 = cy0 + a * (cy1 - cy0) / dist

        ' Get the points P3.
        intersectionx1 = CSng(cx2 + h * (cy1 - cy0) / dist)
        intersectiony1 = CSng(cy2 - h * (cx1 - cx0) / dist)
        intersectionx2 = CSng(cx2 - h * (cy1 - cy0) / dist)
        intersectiony2 = CSng(cy2 + h * (cx1 - cx0) / dist)

        ' See if we have 1 or 2 solutions.
        If (dist = radius0 + radius1) Then
            FindCircleCircleIntersections = 1
        Else
            FindCircleCircleIntersections = 2
        End If
        Exit Function
    End If
End Function

